@@ -13,16 +13,20 @@
 # limitations under the License.
 
 import uuid
+from typing import TYPE_CHECKING, Any, Dict
 
 from selenium.webdriver.remote.remote_connection import RemoteConnection
 
 from appium.common.helper import library_version
 
+if TYPE_CHECKING:
+    from urllib.parse import ParseResult
+
 
 class AppiumConnection(RemoteConnection):
 
     @classmethod
-    def get_remote_connection_headers(cls, parsed_url, keep_alive=True):
+    def get_remote_connection_headers(cls, parsed_url: 'ParseResult', keep_alive: bool = True) -> Dict[str, Any]:
         """Override get_remote_connection_headers in RemoteConnection"""
         headers = RemoteConnection.get_remote_connection_headers(parsed_url, keep_alive=keep_alive)
         headers['User-Agent'] = 'appium/python {} ({})'.format(library_version(), headers['User-Agent'])
