@@ -12,24 +12,32 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, Any, Optional, TypeVar, Union
+
 from selenium import webdriver
 
 from ..mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'HardwareActions'])
+
 
 class HardwareActions(webdriver.Remote):
 
-    def lock(self, seconds=None):
+    def lock(self: T, seconds: Optional[int] = None) -> T:
         """Lock the device. No changes are made if the device is already unlocked.
 
         Args:
-            seconds (optional): The duration to lock the device, in seconds.
+            seconds: The duration to lock the device, in seconds.
                 The device is going to be locked forever until `unlock` is called
                 if it equals or is less than zero, otherwise this call blocks until
                 the timeout expires and unlocks the screen automatically.
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'HardwareActions']: Self instance
         """
         if seconds is None:
             self.execute(Command.LOCK)
@@ -38,40 +46,40 @@ class HardwareActions(webdriver.Remote):
 
         return self
 
-    def unlock(self):
+    def unlock(self: T) -> T:
         """Unlock the device. No changes are made if the device is already locked.
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'HardwareActions']: Self instance
         """
         self.execute(Command.UNLOCK)
         return self
 
-    def is_locked(self):
+    def is_locked(self: T) -> bool:
         """Checks whether the device is locked.
 
         Returns:
-            bool: `True` if the device is locked
+            `True` if the device is locked
         """
         return self.execute(Command.IS_LOCKED)['value']
 
-    def shake(self):
+    def shake(self: T) -> T:
         """Shake the device.
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'HardwareActions']: Self instance
         """
         self.execute(Command.SHAKE)
         return self
 
-    def touch_id(self, match):
+    def touch_id(self: T, match: bool) -> T:
         """Simulate touchId on iOS Simulator
 
         Args:
-            match (bool): Simulates a successful touch (`True`) or a failed touch (`False`)
+            match: Simulates a successful touch (`True`) or a failed touch (`False`)
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'HardwareActions']: Self instance
         """
         data = {
             'match': match
@@ -79,20 +87,20 @@ class HardwareActions(webdriver.Remote):
         self.execute(Command.TOUCH_ID, data)
         return self
 
-    def toggle_touch_id_enrollment(self):
+    def toggle_touch_id_enrollment(self: T) -> T:
         """Toggle enroll touchId on iOS Simulator
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'HardwareActions']: Self instance
         """
         self.execute(Command.TOGGLE_TOUCH_ID_ENROLLMENT)
         return self
 
-    def finger_print(self, finger_id):
+    def finger_print(self: T, finger_id: int) -> Any:
         """Authenticate users by using their finger print scans on supported Android emulators.
 
         Args:
-            finger_id (int): Finger prints stored in Android Keystore system (from 1 to 10)
+            finger_id: Finger prints stored in Android Keystore system (from 1 to 10)
 
         Returns:
             TODO
@@ -100,8 +108,8 @@ class HardwareActions(webdriver.Remote):
         return self.execute(Command.FINGER_PRINT, {'fingerprintId': finger_id})['value']
 
     # pylint: disable=protected-access
-
-    def _addCommands(self):
+    # noinspection PyProtectedMember
+    def _addCommands(self) -> None:
         self.command_executor._commands[Command.LOCK] = \
             ('POST', '/session/$sessionId/appium/device/lock')
         self.command_executor._commands[Command.UNLOCK] = \
