@@ -12,15 +12,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import TYPE_CHECKING, List, TypeVar, Union
+
 from selenium import webdriver
 
 from ..mobilecommand import MobileCommand as Command
 
+if TYPE_CHECKING:
+    # noinspection PyUnresolvedReferences
+    from appium.webdriver.webdriver import WebDriver
+
+T = TypeVar('T', bound=Union['WebDriver', 'IME'])
+
 
 class IME(webdriver.Remote):
 
     @property
-    def available_ime_engines(self):
+    def available_ime_engines(self: T) -> List[str]:
         """Get the available input methods for an Android device.
 
         Package and activity are returned (e.g., ['com.android.inputmethod.latin/.LatinIME'])
@@ -31,26 +39,26 @@ class IME(webdriver.Remote):
         """
         return self.execute(Command.GET_AVAILABLE_IME_ENGINES, {})['value']
 
-    def is_ime_active(self):
+    def is_ime_active(self: T) -> bool:
         """Checks whether the device has IME service active.
         Android only.
 
         Returns:
-            bool: `True` if IME service is active
+            `True` if IME service is active
         """
         return self.execute(Command.IS_IME_ACTIVE, {})['value']
 
-    def activate_ime_engine(self, engine):
+    def activate_ime_engine(self: T, engine: str) -> T:
         """Activates the given IME engine on the device.
 
         Android only.
 
         Args:
-           engine (str): the package and activity of the IME engine to activate
+           engine: the package and activity of the IME engine to activate
                (e.g., 'com.android.inputmethod.latin/.LatinIME')
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'IME']: Self instance
         """
         data = {
             'engine': engine
@@ -58,20 +66,21 @@ class IME(webdriver.Remote):
         self.execute(Command.ACTIVATE_IME_ENGINE, data)
         return self
 
-    def deactivate_ime_engine(self):
+    def deactivate_ime_engine(self: T) -> T:
         """Deactivates the currently active IME engine on the device.
 
         Android only.
 
         Returns:
-            `appium.webdriver.webdriver.WebDriver`
+            Union['WebDriver', 'IME']: Self instance
         """
         self.execute(Command.DEACTIVATE_IME_ENGINE, {})
         return self
 
     @property
-    def active_ime_engine(self):
-        """Returns the activity and package of the currently active IME engine(e.g., 'com.android.inputmethod.latin/.LatinIME').
+    def active_ime_engine(self: T) -> str:
+        """Returns the activity and package of the currently active IME engine
+        (e.g., 'com.android.inputmethod.latin/.LatinIME').
 
         Android only.
 
@@ -81,8 +90,8 @@ class IME(webdriver.Remote):
         return self.execute(Command.GET_ACTIVE_IME_ENGINE, {})['value']
 
     # pylint: disable=protected-access
-
-    def _addCommands(self):
+    # noinspection PyProtectedMember
+    def _addCommands(self) -> None:
         self.command_executor._commands[Command.GET_AVAILABLE_IME_ENGINES] = \
             ('GET', '/session/$sessionId/ime/available_engines')
         self.command_executor._commands[Command.IS_IME_ACTIVE] = \
